#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
//#include <linux/if.h>
#include <net/if.h>
#include <net/ethernet.h>
#include <linux/if_tun.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <vector>


typedef int Socket;

#define max(a,b) ((a)>(b) ? (a):(b))

//The network number has offset 3 bytes in the destination ethernet address
#define NET_OFFSET 3
				 
const uint8_t broadcast_eth_addr[] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
const uint8_t multicast_start_eth_addr[] = {0x01, 0x00, 0x52, 0x00};
//const uint32_t multicast_start_addr = hto

int tap_open (uint8_t devindex) {

	int fd;
	struct ifreq ifr;

	fd = open ("/dev/net/tun", O_RDWR);
	if (fd < 0) {
		fprintf (stderr, "Failed to open /dev/net/tun, err = %d\n", -fd);
		return fd;
	}
	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_flags = IFF_TAP | IFF_NO_PI;
	snprintf (ifr.ifr_name, sizeof (ifr.ifr_name), "tap%u", devindex);
	if (ioctl(fd, TUNSETIFF, (void *) &ifr) < 0) {
	       fprintf(stderr, "TUNSETIFF failed, errno = %d\n", errno);
	       close (fd);
	       return(-errno);
	}
	return (fd);
}

int bindsock(Socket s, in_addr& addr, u_int16_t port, sockaddr_in& sin) {

	memset((char *)&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = port;
	sin.sin_addr = addr;
	if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) == 0)
		return 0;
	else {
		fprintf (stderr, "Could not bind to socket. Error: %d. %s\n", errno, strerror (errno));
		return -1;
	}
}

int connectsock(Socket s, in_addr& addr, u_int16_t port, sockaddr_in& sin) {

	memset((char *)&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = port;
	sin.sin_addr = addr;
	if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) == 0)
		return 0;
	else {
		fprintf (stderr, "Could not connect to socket. Error: %d. %s\n", errno, strerror (errno));
		return -1;
	}
}

typedef std::vector<int32_t> IntArray;

int do_bridge (IntArray& Taps, int rsock, int ssock, uint32_t first_net, uint32_t last_net) {

	const int buffsize = 3100;
	char buff[buffsize];
	fd_set fds;
	int fm = rsock;
	uint32_t tapsize = Taps.size ();
	uint32_t index;
	int nr;
	
	for (index = 0; index < tapsize; index++)
		fm = max (fm, Taps[index]);

	fm++;
	while (1) {
		FD_ZERO (&fds);
		FD_SET (rsock, &fds);
		for (index = 0; index < tapsize; index++)
			FD_SET (Taps[index], &fds);

		if (select (fm, &fds, NULL, NULL, NULL) == -1) {
			fprintf (stderr, "Error %d: %s\n", errno, strerror (errno));
			return -1;
		} else {
			if ((FD_ISSET (rsock, &fds)) && ((nr = read (rsock, buff, buffsize)) >= ETHER_HDR_LEN)) {
				if (memcmp (buff, broadcast_eth_addr, sizeof (broadcast_eth_addr)) == 0)
					for (index = 0; index < tapsize; index++)
						write (Taps[index], buff, nr);
				else {
					uint8_t net_number = *((uint8_t*)(buff + NET_OFFSET));
					if ((net_number < first_net) || (net_number > last_net))
						fprintf (stderr, "Discarding frame to an unknown network %u\n", net_number);
					else
						write (Taps[net_number - first_net], buff, nr);
				}
			}

			for (index = 0; index < tapsize; index++)
				if ((FD_ISSET (Taps[index], &fds)) && ((nr = read (Taps[index], buff, buffsize)) >= ETHER_HDR_LEN))
					write (ssock, buff, nr);
		}
	}
	return 0;
}

int main (int argc, char** argv) {

	uint32_t first_net, last_net, first_tap, last_tap, index;
	
	Socket rsock, ssock;
        struct in_addr addr;
	uint16_t port;
	sockaddr_in sin;

	if (argc < 9) {
		fprintf (stderr, "Usage: %s <tap range> <networks range> <recv IP recv port> <send IP send port>\n", argv[0]);
		return -1;
	}

	first_tap = atoi (argv[1]);
	last_tap = atoi (argv[2]);
	first_net = atoi (argv[3]);
	last_net = atoi (argv[4]);

	//Sanity check
	if (((last_net - first_net) < 0) || ((last_tap - first_tap) < 0)
		|| ((last_net - first_net) != (last_tap - first_tap))) {
		fprintf (stderr, "Incorrect tap and network ranges\n");
		return -1;
	}

	IntArray Taps (last_tap - first_tap + 1);
	for (index = 0; index < last_tap - first_tap + 1; index++) {
		Taps[index] = tap_open (first_tap + index);
		if (Taps[index] < 0)
			return -1;
		else
			ioctl (Taps[index], TUNSETNOCSUM, 1);
	}

	rsock = socket (AF_INET, SOCK_DGRAM, 0);
	ssock = socket (AF_INET, SOCK_DGRAM, 0);
	if ((rsock < 0) || (ssock < 0)) {
		fprintf (stderr, "Could not open socket\n");
		return -1;
	}

	inet_aton (argv[5], &addr);
        port = htons(atoi(argv[6]));
	if (bindsock (rsock, addr, port, sin) == -1)
		return -1;
	
	inet_aton (argv[7], &addr);
        port = htons(atoi(argv[8]));
	if (connectsock (ssock, addr, port, sin) == -1)
		return -1;
	
	return (do_bridge (Taps, rsock, ssock, first_net, last_net));
	
/*	if ((rsock > 0) && (ssock > 0)) {
		ioctl(tapf, TUNSETNOCSUM, 1);
		fm = max (tapf, rsock) + 1; 
		while (1) {
			FD_ZERO (&fds);
			FD_SET (tapf, &fds);
			FD_SET (rsock, &fds);

			select (fm, &fds, NULL, NULL, NULL);
			if (FD_ISSET (tapf, &fds)) {
				nr = read (tapf, buff, buffsize);
				write (ssock, buff, nr);
			}

			if (FD_ISSET (rsock, &fds)) {
				nr = read (rsock, buff, buffsize);
				write (tapf, buff, nr);
			}
*/
/*			if (select (fm, &fds, NULL, NULL, NULL) > 0) {

				if ((FD_ISSET (tapf, &fds)) && ((nr = read (tapf, buff, buffsize)) > 0))
					write (f2, buff, nr);

				if ((FD_ISSET (f2, &fds)) && ((nr = read (f2, buff, buffsize)) > 0))
					write (tapf, buff, nr);
			}

		}
	} else
		return -1;
		*/
}
