# An example script for the usage of ns-2 in emulation mode with distributed applications: 
# Uses:
# 1. Network/IP/UDP agents to access to a network device at level II
# 2. Tap/Coop agents to cooperatively use a Network object
# 3. In-memory compression of the trace file to reduce disk write operations during simulation


# Common variables
# 

set val(chan)           Channel/WirelessChannel    ;# Channel Type
set val(prop)           Propagation/TwoRayGround   ;# radio-propagation model
set val(netif)          Phy/WirelessPhy            ;# network interface type
set val(mac)            Mac/802_11                 ;# MAC type
set val(ifq)            Queue/DropTail/PriQueue    ;# interface queue type
set val(ll)             LL                         ;# link layer type
set val(ant)            Antenna/OmniAntenna        ;# antenna model
set val(ifqlen)         50                         ;# max packet in ifq
set val(x)              100			   ;# x range in meters
set val(y)              100			   ;# y range in meters
#set val(rp)		AODV			   ;# routing protocol
set val(rp)		DumbAgent		   ;# routing protocol
set val(nn)		6			   ;# number of mobile nodes
#set val(stime)		36000.0			   ;# simulation time
set val(stime)		30.0			   ;# simulation time

set ns		[new Simulator]
$ns use-scheduler RealTime

set tracefd [open ns2emu-dstapp.tr w]
$ns trace-all $tracefd
set tr0 [new BaseTrace/ShmGZ ns2emu-dstapp.tr.gz 16 400 104857600]

set topo [new Topography]

$topo load_flatgrid $val(x) $val(y)

# Create GOD
create-god $val(nn)

# Create channel
set chan_1_ [new $val(chan)]

# Configure nodes
$ns node-config -adhocRouting $val(rp) \
	-llType $val(ll) \
	-macType $val(mac) \
	-ifqType $val(ifq) \
	-ifqLen $val(ifqlen) \
	-antType $val(ant) \
	-propType $val(prop) \
	-phyType $val(netif) \
	-topoInstance $topo \
	-agentTrace ON \
	-routerTrace ON \
	-macTrace ON \
	-movementTrace OFF \
	-channel $chan_1_

$ns set-address-format hierarchical 2 24 8

proc setup_node {id x y z color} {
	
#Procedure to configure an ns-2 node initially
	global ns node_
	set node_($id) [$ns node $id]
	$node_($id) set X_ $x
	$node_($id) set Y_ $y
	$node_($id) set Z_ $z
	$node_($id) color $color
	$ns at 0 "$node_($id) setdest $x $y 0"
	$ns at 0 "$node_($id) color $color"
	$node_($id) random-motion 0
}
				
setup_node 1.2 10 20 0 "red"
setup_node 1.3 10 30 0 "red"
setup_node 2.2 20 20 0 "red"
setup_node 2.3 20 30 0 "red"
setup_node 3.2 30 20 0 "red"
setup_node 3.3 30 30 0 "red"
					
#Network object IPUDP
set ipudp2 [new Network/IP/UDP]
$ipudp2 open readwrite
$ipudp2 bind 192.168.1.1 10002
$ipudp2 connect 192.168.1.2 10000

set ipudp3 [new Network/IP/UDP]
$ipudp3 open readwrite
$ipudp3 bind 192.168.1.1 10003
$ipudp3 connect 192.168.1.3 10000

#Tap Agent for each node
Agent/Tap/Coop set maxpkt_ 3100
set a12 [new Agent/Tap/Coop 1.2]
set a13 [new Agent/Tap/Coop 1.3]
set a22 [new Agent/Tap/Coop 2.2]
set a23 [new Agent/Tap/Coop 2.3]
set a32 [new Agent/Tap/Coop 3.2]
set a33 [new Agent/Tap/Coop 3.3]

source "pdelays.tcl"
puts "install nets into taps..."
#Assign network objects to TAP agents
$a12 network $ipudp2
$a22 network $ipudp2
$a32 network $ipudp2

$a13 network $ipudp3
$a23 network $ipudp3
$a33 network $ipudp3

#Assign TAP agents to ns-2 nodes
$ns attach-agent $node_(1.2) $a12
$ns attach-agent $node_(1.3) $a13
$ns attach-agent $node_(2.2) $a22
$ns attach-agent $node_(2.3) $a23
$ns attach-agent $node_(3.2) $a32
$ns attach-agent $node_(3.3) $a33
        
$ns at $val(stime) "stop"
$ns at $val(stime) "puts \"NS EXITING ...\" ; $ns halt"

proc stop {} {
	global ns tracefd ipudp2 ipudp3
	$ns flush-trace
	close $tracefd
	$ipudp2 close
	$ipudp3 close
}
	    
puts "okey"
	
$ns run
